﻿from ctypes import Structure, Union
from ctypes.wintypes import HANDLE, BOOL, DWORD, ULONG, LONG, LARGE_INTEGER, BYTE, LPVOID, POINTER
import os.path
from win32con import GENERIC_WRITE, GENERIC_READ, FILE_BEGIN

#from liveusb.creator import LiveUSBError

class STORAGE_DEVICE_NUMBER(Structure):
    _fields_ = [
            ("DeviceType", DWORD), # DEVICE_TYPE in Win32 API
            ("DeviceNumber", ULONG),
            ("PartitionNumber", ULONG)]

def _getPhysicalDeviceId(device):
    from win32file import CreateFile
    from win32con import OPEN_EXISTING
    from winioctlcon import IOCTL_STORAGE_GET_DEVICE_NUMBER
    from ctypes import windll, byref, sizeof

    hdevice = CreateFile(device, 0, 0, None, OPEN_EXISTING, 0, None)
    if hdevice < 0:
        raise LiveUSBError(_("Another program is currently accessing the device. Close all other programs and try again."))
    buf = STORAGE_DEVICE_NUMBER()
    returned = DWORD()
    res = windll.kernel32.DeviceIoControl(hdevice.handle,
            IOCTL_STORAGE_GET_DEVICE_NUMBER,
            None, 0,
            byref(buf), sizeof(buf),
            byref(returned),
            None)
    if not res:
        raise LiveUSBError(_("Another program is currently accessing the device. Close all other programs and try again."))
    hdevice.Close()
    return buf.DeviceNumber

def _getHandleOnVolume(volumepath, access):
    from win32file import CreateFile
    from win32con import FILE_SHARE_READ, FILE_SHARE_WRITE, OPEN_EXISTING
    return CreateFile(volumepath, access, FILE_SHARE_READ|FILE_SHARE_WRITE, None, OPEN_EXISTING, 0, None)

def _getHandleOnFile(filepath, access):
    from win32file import CreateFile
    from win32con import GENERIC_READ, OPEN_EXISTING, CREATE_ALWAYS, FILE_FLAG_SEQUENTIAL_SCAN

    disposition = CREATE_ALWAYS
    if access == GENERIC_READ:
        disposition = OPEN_EXISTING

    return CreateFile("\\\\.\\" + filepath, access, 0, None, disposition, FILE_FLAG_SEQUENTIAL_SCAN, None)

def _getHandleOnDevice(device, access):
    from win32file import CreateFile
    from win32con import FILE_SHARE_READ, FILE_SHARE_WRITE, OPEN_EXISTING, FILE_FLAG_WRITE_THROUGH, FILE_FLAG_NO_BUFFERING
    return CreateFile("\\\\.\\PhysicalDrive%d" % device, access, FILE_SHARE_READ | FILE_SHARE_WRITE, None, OPEN_EXISTING, FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, None)
def _getLockOnVolumeByHandle(handle):
    from winioctlcon import FSCTL_LOCK_VOLUME
    from ctypes import windll, byref

    returned = DWORD()
    return windll.kernel32.DeviceIoControl(int(handle), FSCTL_LOCK_VOLUME, None, 0, None, 0, byref(returned), None)

def _removeLockOnVolumeByHandle(handle):
    from winioctlcon import FSCTL_UNLOCK_VOLUME
    from ctypes import windll, byref

    returned = DWORD()
    return windll.kernel32.DeviceIoControl(int(handle), FSCTL_UNLOCK_VOLUME, None, 0, None, 0, byref(returned), None)

def _unmountVolumeByHandle(handle):
    from winioctlcon import FSCTL_DISMOUNT_VOLUME
    from ctypes import windll, byref

    returned = DWORD()
    return windll.kernel32.DeviceIoControl(int(handle), FSCTL_DISMOUNT_VOLUME, None, 0, None, 0, byref(returned), None)
    
def _isVolumeMountedByHandle(handle):
    from winioctlcon import FSCTL_IS_VOLUME_MOUNTED
    from ctypes import windll, byref

    returned = DWORD()
    return windll.kernel32.DeviceIoControl(int(handle), FSCTL_IS_VOLUME_MOUNTED, None, 0, None, 0, byref(returned), None)

class DISK_GEOMETRY(Structure):
    _fields_ = [
            ("Cylinders", LARGE_INTEGER),
            ("MediaType", BYTE), # MEDIA_TYPE in Win32 API
            ("TracksPerCylinder", DWORD),
            ("SectorsPerTrack", DWORD),
            ("BytesPerSector", DWORD)]

class OVERLAPPED(Structure):
    pass

def _getNumberOfSectorsAndSectorSizeByHandle(handle):
    from ctypes import windll, byref, WinError, sizeof
    from winioctlcon import IOCTL_DISK_GET_DRIVE_GEOMETRY

    geometry = DISK_GEOMETRY()
    returned = DWORD()
    res = windll.kernel32.DeviceIoControl(int(handle),
            IOCTL_DISK_GET_DRIVE_GEOMETRY,
            None, 0,
            byref(geometry), sizeof(geometry),
            byref(returned),
            None)
    if not res:
        raise WinError()
    return geometry.Cylinders * geometry.TracksPerCylinder * geometry.SectorsPerTrack, geometry.BytesPerSector

if __name__ == "__main__":
    vol = "\\\\.\\H:"
    d = _getPhysicalDeviceId(vol)
    hvol = _getHandleOnVolume(vol, GENERIC_WRITE)
    print _getLockOnVolumeByHandle(hvol)
    print _unmountVolumeByHandle(hvol)
    hd = _getHandleOnDevice(d, GENERIC_WRITE)
    (numsects, sectorsize) =  _getNumberOfSectorsAndSectorSizeByHandle(hd)
    print (numsects, sectorsize)
    hd.Close()
    file = "d:\\Documents and Settings\\Benjamin\\_vimrc"
    hd = open("\\\\.\\PhysicalDrive%d" % d, 'rb+', 0)
#     hfile = _getHandleOnFile(file, GENERIC_READ)
    hfile = open(file, 'rb', 0)
#     _seek(hfile, 0)
    hfile.seek(0)
#     data = _read(hfile, sectorsize, 1)
    data = hfile.read(sectorsize)
    print data
    hd.seek(0)
    hd.write(data)
#     _write(hd, data, sectorsize, 1)
#     hd = _getHandleOnDevice(d, GENERIC_READ)
    hd.seek(0)
    data = hd.read(sectorsize)
#     data2 = _read(hd, sectorsize, 1)
#     print repr(data2.value)
    print data
