from setuptools import setup
import sys, os
import shutil

shutil.rmtree("build", ignore_errors=True)
shutil.rmtree("dist", ignore_errors=True)

LOCALE_DIR= '/usr/share/locale'

version = '1.2.0'
locales = []
if os.path.exists('po/locale'):
    for lang in os.listdir('po/locale'):
        locales.append(os.path.join(lang, 'LC_MESSAGES'))

common = dict(
        name = 'Jolicloud USB Creator',
        version = version,
        packages = ['liveusb'],
        license = 'GNU General Public License (GPL)',
        url = 'http://git.jolicloud.org/?p=projects/jolicloud-usb-creator.git',
        description = 'This tool dumps hybrid ISO on USB stick',
        long_description = 'Jolicloud USB writer is a Linux/Windows/Mac OS X tool for easily dump hybrid ISO on USB flash drives',
        maintainer = 'Benjamin Ryzman',
        maintainer_email = 'benjamin@jolicloud.com',
)

if sys.platform == 'win32':
    import py2exe

    ################################################################
    import os

    class InnoScript:
        def __init__(self,
                     name,
                     lib_dir,
                     dist_dir,
                     windows_exe_files = [],
                     lib_files = [],
                     version = "1.0"):
            self.lib_dir = lib_dir
            self.dist_dir = dist_dir
            if not self.dist_dir[-1] in "\\/":
                self.dist_dir += "\\"
            self.name = name
            self.version = version
            self.windows_exe_files = [self.chop(p) for p in windows_exe_files]
            self.lib_files = [self.chop(p) for p in lib_files]

        def chop(self, pathname):
            assert pathname.startswith(self.dist_dir)
            return pathname[len(self.dist_dir):]
        
        def create(self, pathname="dist\\jolicloud-usb-creator.iss"):
            self.pathname = pathname
            ofi = self.file = open(pathname, "w")
            print >> ofi, "; WARNING: This script has been created by py2exe. Changes to this script"
            print >> ofi, "; will be overwritten the next time py2exe is run!"
            print >> ofi, r"[Setup]"
            print >> ofi, r"AppName=%s" % self.name
            print >> ofi, r"AppVerName=%s %s" % (self.name, self.version)
            print >> ofi, r"AppVersion=%s" % self.version
            print >> ofi, r"VersionInfoVersion=%s" % self.version
            print >> ofi, r"AppPublisher=Jolicloud"
            print >> ofi, r"AppCopyright=Copyright (C) 2009 Jolicloud"
            print >> ofi, r"SetupIconFile=..\data\icons\juc.ico"
            print >> ofi, r"AppPublisherURL=http://www.jolicloud.com/"
            print >> ofi, r"DefaultDirName={pf}\%s" % self.name
            print >> ofi, r"DefaultGroupName=%s" % self.name
            print >> ofi, r"OutputBaseFilename=%s Installer %s" % (self.name, self.version)
            print >> ofi

            print >> ofi, r"[Files]"
            for path in self.windows_exe_files + self.lib_files:
                print >> ofi, r'Source: "%s"; DestDir: "{app}\%s"; Flags: ignoreversion' % (path, os.path.dirname(path))
            print >> ofi

            print >> ofi, r"[Icons]"
            for path in self.windows_exe_files:
                print >> ofi, r'Name: "{group}\%s"; Filename: "{app}\%s"; WorkingDir: "{app}"' % \
                      (self.name, path)
            print >> ofi, 'Name: "{group}\Uninstall %s"; Filename: "{uninstallexe}"' % self.name

        def compile(self):
            try:
                import ctypes
            except ImportError:
                try:
                    import win32api
                except ImportError:
                    import os
                    os.startfile(self.pathname)
                else:
                    print "Ok, using win32api."
                    win32api.ShellExecute(0, "compile",
                                                    self.pathname,
                                                    None,
                                                    None,
                                                    0)
            else:
                print "Cool, you have ctypes installed."
                res = ctypes.windll.shell32.ShellExecuteA(0, "compile",
                                                          self.pathname,
                                                          None,
                                                          None,
                                                          0)
                if res < 32:
                    raise RuntimeError, "ShellExecute failed, error %d" % res


    ################################################################

    from py2exe.build_exe import py2exe

    class build_installer(py2exe):
        # This class first builds the exe file(s), then creates a Windows installer.
        # You need InnoSetup for it.
        def run(self):
            # First, let py2exe do it's work.
            py2exe.run(self)

            lib_dir = self.lib_dir
            dist_dir = self.dist_dir
            self.lib_files = filter(lambda f: f.find(r'\build\bdist') < 0, self.lib_files)
            
            # create the Installer, using the files py2exe has created.
            script = InnoScript("Jolicloud USB Creator",
                                lib_dir,
                                dist_dir,
                                self.windows_exe_files,
                                self.lib_files, version)
            print "*** creating the inno setup script***"
            script.create()
            print "*** compiling the inno setup script***"
            script.compile()
            # Note: By default the final setup.exe will be in an Output subdirectory.

    ###############################################################

    try:
        import modulefinder
        import win32com
        for p in win32com.__path__[1:]: # Take the path to win32comext
            modulefinder.AddPackagePath("win32com", p)
        pn = "win32com.shell"
        __import__(pn)
        m = sys.modules[pn]
        for p in m.__path__[1:]:
            modulefinder.AddPackagePath(pn, p)
    except ImportError:
        pass

    LOCALE_DIR = 'locale'

    setup(
        platforms = ['Windows'], 
        setup_requires=['py2exe'],
        app=['jolicloud-usb-creator.py'],
        windows = [
            {
                "script" : "jolicloud-usb-creator.py",
                "icon_resources" : [(0, "data/icons/juc.ico")],
                "uac_info" : "requireAdministrator"
            }
        ],
        options = {
            "py2exe": {
                'bundle_files': 3,
                'compressed': 1,
                'optimize': 2,
                "includes": ["sip"]
            }
        }, 
        cmdclass = {"py2exe": build_installer},
        zipfile=None,
        data_files = [ "LICENSE.txt" ] + [(os.path.join(LOCALE_DIR, locale),
                [os.path.join('po', 'locale', locale, 'jolicloud-usb-creator.mo')])
                    for locale in locales],
        **common
    )
elif sys.platform.startswith("linux"):
    setup(
        platforms = ['Linux'],
        scripts = ['jolicloud-usb-creator.py'], 
        data_files = [
            ("/usr/share/applications",["data/jolicloud-usb-creator.desktop"]), 
            ("/usr/share/icons/hicolor/128x128/apps",["data/icons/usb-creator-icon-128.png"]),
            ("/usr/share/icons/hicolor/64x64/apps",["data/icons/usb-creator-icon-64.png"]),
            ("/usr/share/icons/hicolor/48x48/apps",["data/icons/usb-creator-icon-48.png"]),
            ("/usr/share/icons/hicolor/32x32/apps",["data/icons/usb-creator-icon-32.png"]),
        ] + [(os.path.join(LOCALE_DIR, locale),
                [os.path.join('po', 'locale', locale, 'jolicloud-usb-creator.mo')])
                    for locale in locales],
        **common
    )
elif sys.platform == 'darwin':
    setup(
        setup_requires=['py2app'],
        platforms = ['Mac OS X'],
        app = ['jolicloud-usb-creator.py'],
        options={
            "py2app": {
                "argv_emulation": True,
                "includes": ["sip"],
                #"iconfile": "data/icons/jolicloud-usb-creator.icns"
            }
        },
        data_files = [
            "LICENSE.txt",
            "qt.conf",
        ] + [(os.path.join(LOCALE_DIR, locale),
                [os.path.join('po', 'locale', locale, 'jolicloud-usb-creator.mo')])
                    for locale in locales],
        **common
        )
